package whiteship.crud.presentation {

	import flash.events.Event;
	import flash.events.EventDispatcher;
	
	import mx.collections.ArrayCollection;
	import mx.controls.Alert;
	
	import org.as3commons.lang.Assert;
	import org.springextensions.actionscript.core.operation.IOperation;
	import org.springextensions.actionscript.core.operation.OperationEvent;
	import whiteship.crud.domain.Todo;
	import whiteship.crud.domain.service.ITodoService;

	public class TodoListPresentationModel extends EventDispatcher {

		// --------------------------------------------------------------------
		//
		// Private Variables
		//
		// --------------------------------------------------------------------

		private var _todoService:ITodoService;

		// --------------------------------------------------------------------
		//
		// Constructor
		//
		// --------------------------------------------------------------------

		public function TodoListPresentationModel(todoService:ITodoService) {
			Assert.notNull(todoService, "The todo service must not be null");
			_todoService = todoService;
			loadTodos();
		}

		// --------------------------------------------------------------------
		//
		// Public Properties
		//
		// --------------------------------------------------------------------

		// ----------------------------
		// todos
		// ----------------------------

		private var m_todos:ArrayCollection;

		[Bindable(event="todosChange")]
		public function get todos():ArrayCollection {
			return m_todos;
		}

		public function set todos(value:ArrayCollection):void {
			if (value !== m_todos) {
				m_todos = value;
				dispatchEvent(new Event("todosChange"));
			}
		}

		// ----------------------------
		// selectedTodo
		// ----------------------------

		private var m_selectedTodo:Todo;

		[Bindable(event="selectedTodoChange")]
		public function get selectedTodo():Todo {
			return m_selectedTodo;
		}

		public function set selectedTodo(value:Todo):void {
			if (value !== m_selectedTodo) {
				m_selectedTodo = value;
				dispatchEvent(new Event("selectedTodoChange"));
			}
		}

		// ----------------------------
		// todoDetailsPresentationModel
		// ----------------------------

		private var m_todoDetailsPresentationModel:TodoDetailPresentationModel;

		[Bindable(event="todoDetailsPresentationModelChange")]
		public function get todoDetailsPresentationModel():TodoDetailPresentationModel {
			return m_todoDetailsPresentationModel;
		}

		public function set todoDetailsPresentationModel(value:TodoDetailPresentationModel):void {
			if (value !== m_todoDetailsPresentationModel) {
				m_todoDetailsPresentationModel = value;
				dispatchEvent(new Event("todoDetailsPresentationModelChange"));
			}
		}

		// ----------------------------
		// editAllowed
		// ----------------------------

		[Bindable(event="selectedTodoChange")]
		[Bindable(event="deletingChange")]
		[Bindable(event="savingChange")]
		public function get editAllowed():Boolean {
			return (!deleting && !saving && (m_selectedTodo != null));
		}

		// ----------------------------
		// deleteAllowed
		// ----------------------------

		[Bindable(event="selectedTodoChange")]
		[Bindable(event="deletingChange")]
		[Bindable(event="savingChange")]
		public function get deleteAllowed():Boolean {
			return (!deleting && !saving && (m_selectedTodo != null));
		}
		
		// ----------------------------
		// deleting
		// ----------------------------
		
		private var m_deleting:Boolean;
		
		[Bindable(event="deletingChange")]
		private function get deleting():Boolean {
			return m_deleting;
		}
		
		private function set deleting(value:Boolean):void {
			if (value !== m_deleting) {
				m_deleting = value;
				dispatchEvent(new Event("deletingChange"));
			}
		}
		
		// ----------------------------
		// saving
		// ----------------------------
		
		private var m_saving:Boolean;
		
		[Bindable(event="savingChange")]
		private function get saving():Boolean {
			return m_saving;
		}
		
		private function set saving(value:Boolean):void {
			if (value !== m_saving) {
				m_saving = value;
				dispatchEvent(new Event("savingChange"));
			}
		}

		// --------------------------------------------------------------------
		//
		// Public Methods
		//
		// --------------------------------------------------------------------

		public function addTodo():void {
			todoDetailsPresentationModel = createDetailsPresentationModel();
		}

		public function deleteSelectedTodo():void {
			deleting = true;
			
			var operation:IOperation = _todoService.deleteTodo(m_selectedTodo);
			operation.addCompleteListener(deleteTodo_completeHandler);
		}

		public function editSelectedTodo():void {
			todoDetailsPresentationModel = createDetailsPresentationModel(m_selectedTodo);
		}

		// --------------------------------------------------------------------
		//
		// Private Methods
		//
		// --------------------------------------------------------------------

		private function loadTodos():void {
			var operation:IOperation = _todoService.getTodos();
			operation.addCompleteListener(loadTodos_completeHandler);
			operation.addErrorListener(loadTodos_errorHandler);
		}

		private function loadTodos_completeHandler(event:OperationEvent):void {
			todos = event.result;
		}

		private function loadTodos_errorHandler(event:OperationEvent):void {
			Alert.show("There was an error loading the list of todos.", "Application Error");
		}

		private function createDetailsPresentationModel(todo:Todo = null):TodoDetailPresentationModel {
			var result:TodoDetailPresentationModel = new TodoDetailPresentationModel(todo);
			result.addEventListener(TodoDetailPresentationModel.COMMIT, todoDetailsPresentationModel_commitHandler);
			result.addEventListener(TodoDetailPresentationModel.CANCEL, todoDetailsPresentationModel_cancelHandler);
			return result;
		}

		private function todoDetailsPresentationModel_commitHandler(event:Event):void {
			var todo:Todo = todoDetailsPresentationModel.todo;
			var operation:IOperation = _todoService.saveTodo(todo);
			operation.addCompleteListener(saveTodo_completeHandler);
			
			saving = true;
			todoDetailsPresentationModel = null;
		}

		private function todoDetailsPresentationModel_cancelHandler(event:Event):void {
			todoDetailsPresentationModel = null;
		}

		private function deleteTodo_completeHandler(event:OperationEvent):void {
			deleting = false;
			selectedTodo = null;
		}
		
		private function saveTodo_completeHandler(event:OperationEvent):void {
			saving = false;
		}
		
	}
}